/*
 * Startup Code for ARM
 * ARM-Defines for Assembler
 *
 * by: Clemens Helfmeier
 * ClemensHelfmeier {at] gmx `dot" de
 * (replace the corresponding at and dot in the email, please).
 */

#ifndef _ARM_INC_
#define _ARM_INC_

//
// ******************************************************************************
//                ARM related constants
// ******************************************************************************
#define ARM_MODE_User		(0x10)
#define ARM_MODE_FIQ		(0x11)
#define ARM_MODE_IRQ		(0x12)
#define ARM_MODE_Supervisor	(0x13)
#define ARM_MODE_Abort		(0x17)
#define ARM_MODE_Undefined	(0x1B)
#define ARM_MODE_System		(0x1F)

#define ARM_I_BIT		(0x80)
#define ARM_F_BIT		(0x40)
#define ARM_T_BIT		(0x20)

#if defined(ASSEMBLER)
/* the following defines are always assembled in ARM-Mode.
 * the function can be called either in ARM, THUMB or THUMB-Interwork
 */
#if defined(ARM_MODE_THUMB)
	/* all C-Functions are compiled without Thumb-Interwork.
	 * So make shure that Interwork is supported by the Caller!
	 */
	.macro CALL_FUNC	func
		ldr	r12, =\func
		mov	lr, pc
		b	CALLSTUB_THUMB
	.endm
	.macro CALL_REG		reg
		mov 	r12, \reg
		mov	lr, pc
		b	CALLSTUB_THUMB
	.endm
#elif defined(ARM_MODE_THUMB_INTERWORK)
	/* all C-Functions are compiled with Thumb-Interwork.
	 * calling can be done by bx, return is also always bx */
	.macro CALL_FUNC	func
		ldr	r12, =\func
		mov	lr, pc
		bx	r12
	.endm
	.macro CALL_REG		reg
		mov	lr, pc
		bx	\reg
	.endm
#else
	/* C-Functions are compiled in ARM-Mode */
	.macro CALL_FUNC	func
		bl	\func
	.endm
	.macro CALL_REG		reg
		/* indirect call only via BX possible on armv4T */
		mov	lr, pc
		bx	\reg
	.endm
#endif
#endif /* defined(ASSEMBLER) */

#endif
