/*
 * Startup Code for ARM
 *
 * Main File.
 * Contains a very simple main routine and an interrupt handler.
 *
 * by: Clemens Helfmeier
 * ClemensHelfmeier {at] gmx `dot" de
 * (replace the corresponding at and dot in the email, please).
 */


#include "AT91SAM7S64.h"
#define __inline extern inline
#include "lib_AT91SAM7S64.h"


volatile unsigned int j = 0;

void sys_handler( void );

void TestCallRecursive( int bar );
void TestCallCPP( void );

/* main()
 * After initialization has been done, this routine will be called.
 * There are no arguments, the routine should never exit!
 */
int main( void ) {

	/* enable AIC-Clock */
	AT91F_AIC_CfgPMC();
/* we want an interrupt to happen -> Periodic Interval Timer (PIT, PITC) */
	/* configure the AIC for PIT */
	AT91F_AIC_ConfigureIt( AT91C_BASE_AIC, AT91C_ID_SYS, 0 /*priority lowest*/, AT91C_AIC_SRCTYPE_INT_EDGE_TRIGGERED, sys_handler );

	/* enable the timer interrupt */
	AT91C_BASE_PITC->PITC_PIMR = AT91C_SYSC_PITEN | AT91C_SYSC_PITIEN | 50;
	/* clear timer by reading the CPIV */
	{ volatile unsigned int i = AT91C_BASE_PITC->PITC_PIVR; }
	/* enable the interupt vector */
	AT91F_AIC_EnableIt( AT91C_BASE_AIC, AT91C_ID_SYS );

/* we want to measure the pll frequency -> put it out on PCK0 divided by 64 */
	AT91F_PIO_CfgPeriph( AT91C_BASE_PIOA, 0x00, 1<<6 );
	AT91C_BASE_PMC->PMC_PCKR[0] = AT91C_PMC_CSS_PLL_CLK | AT91C_PMC_PRES_CLK_64;
	AT91C_BASE_PMC->PMC_SCER = AT91C_PMC_PCK0;

	AT91F_PIO_CfgOutput( AT91C_BASE_PIOA, 1<<1 );
	AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, 1<<1 );

/* do some example test calls */
	TestCallRecursive( 100 );
	TestCallCPP();

/* loop forever. */
	while (1);
}
void TestCallRecursive( int bar ) {
	if (bar > 0) TestCallRecursive(bar-1);
	j++;
}
void sys_handler( void ) {
	/* the System interrupt handler must lookup the source of the interrupt.
	 * this currently can only be the PIT, but have a look: */
	if (AT91C_BASE_PITC->PITC_PISR != 0x00) {
		/* the periodic interval timer has expired */
		if ((AT91F_PIO_GetOutputStatus( AT91C_BASE_PIOA ) & (1<<1)) != 0)
			AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, 1<<1 );
		else
			AT91F_PIO_SetOutput( AT91C_BASE_PIOA, 1<<1 );

	}
}
